/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This examples demonstrates how to build PDUs for encoding. It exersizes
 * various IAAPIs needed to construct PDUs for encoding using SEQUENCE as
 * a PDU (see test.asn).
 */



#include "test.h"
#include "iaapi.h"
 




int main(void)
{

    TypeHndl th;
    int ix;       
    IAAPI_ERRTYPE err;
    int ret;
    void *comp;
    OssGlobal w, *world = &w;
    OssBuf encodedData = {0};
    PDU *myPDU, *decodedData;
    int pdu_num = 0;


    ret = ossinit(world, test);
    if (ret)
	printf("Initialization failed with error = %d\n", ret);

    ossSetIaapiErrorHandlingFunction(world, ossDefaultIAAPI_ERR);

    myPDU = malloc(sizeof(PDU));
    memset(myPDU, 0, sizeof(PDU));

    th = ossTypeHandleOfPDU(world, PDU_PDU);
    ossPrint(world, "\tPDU ASN.1 notation...\n\n");
    ossPrintASN1DescriptionOfType(world, th, FALSE);
        
    ossPrint(world, "\n\tUnconstructed original PDU...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, myPDU);
    if (ret) {
	ossterm(world);
	return ret;
    }

    ossPrint(world, "\nConstructing the PDU with IAAPI...\n");
    ix = ossItemIndexByName(world, th, "data1");
    err = ossPutSimpleTypeValue(world,
		ossTypeHandleOfComponent(world, th, ix), "4711", &comp);
    err = ossPutStructTypeValue(world, th, comp, ix, (void **)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

    ix = ossItemIndexByName(world, th, "data2");
    err = ossPutSimpleTypeValue(world,
		ossTypeHandleOfComponent(world, th, ix), "4712", &comp);
    err = ossPutStructTypeValue(world, th, comp, ix, (void **)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

    ix = ossItemIndexByName(world, th, "data3");
    err = ossPutSimpleTypeValue(world,
		ossTypeHandleOfComponent(world, th, ix), "4713", &comp);
    err = ossPutStructTypeValue(world, th, comp, ix, (void **)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

    ossPrint(world, "\n\tPDU constructed with IAAPI...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, myPDU);
    if (ret) {
	ossterm(world);
	return ret;
    }

    ossSetEncodingRules(world, OSS_BER);
    ossSetFlags(world, DEBUGPDU);
    ossPrint(world, "\nEncoding the constructed PDU...\n\n");
    if (ret = ossEncode(world, PDU_PDU, myPDU, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));
    else {
        ossPrint(world, "\n\tEncoded constructed PDU...\n\n");
        ossPrintHex(world, encodedData.value, encodedData.length);
	decodedData = NULL;
	ossPrint(world, "\nDecoding the PDU...\n\n");
	if (ret = ossDecode(world, &pdu_num, &encodedData, (void **)&decodedData))
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decodedData);
	    ossFreePDU(world, pdu_num, decodedData);
	}
	ossFreeBuf(world, encodedData.value);
    }

    ossFreePDU(world, PDU_PDU, myPDU);
    ossterm(world);
    return ret;
}
