/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This examples demonstrates how to build PDUs for encoding. It exersizes
 * various IAAPIs needed to construct PDUs for encoding using SEQUENCE,
 * SEQUENCE OF and OpenType values (see test3.asn).
 */


#include "test3.h"
#include "iaapi.h"

static int buildUnencodedPDU(OssGlobal *world, int pdu_num, void **decodedData);





int main(void)
{

    TypeHndl pduth, idth, ath, otth;
    unsigned int idx, ax, otx;
    IAAPI_ERRTYPE err;
    int ret;
    void *idcomp, *acomp, *otcomp;
    OssGlobal w, *world = &w;
    OssBuf encodedData = {0};
    PDU *myPDU, *decodedData;
    int pdu_num = 0;


    ret = ossinit(world, test3);
    if (ret)
	printf("Initialization failed with error = %d\n", ret);

    ossSetIaapiErrorHandlingFunction(world, ossDefaultIAAPI_ERR);

	/*
	 * Get the type handle of the CHOICE PDU.
	 */
    pduth = ossTypeHandleOfPDU(world, PDU_PDU);
    ossPrint(world, "\tPDU ASN.1 notation...\n\n");
    ossPrintASN1DescriptionOfType(world, pduth, FALSE);

	/*
	 * Construct another PDU using ossPutEncodedValue().
	 */
    ossPrint(world, "\nConstructing the PDU with IAAPI...\n");

	/*
	 * Get the index and the type handle of the Attribute component of
	 * the CHOICE PDU.
	 */
    ax = ossItemIndexByName(world, pduth, "pdu3");
    ath = ossTypeHandleOfComponent(world, pduth, ax);

	/*
	 * Get the index and the type handle of the "id" component of
	 * the Attribute component of the CHOICE PDU.
	 */
    idx = ossItemIndexByName(world, ath, "id");
    idth = ossTypeHandleOfComponent(world, ath, idx);
	/*
	 * Create an "id" component of the Attribute component of
	 * the CHOICE PDU.
	 */
    idcomp = NULL;
    err = ossPutSimpleTypeValue(world, idth, "1", &idcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create an Attribute component of the CHOICE PDU with the
	 * "id" component.
	 */
    acomp = NULL;
    err = ossPutStructTypeValue(world, ath, idcomp, idx, &acomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Get the index and the type handle of the OpenType component of
	 * the Attribute component of the CHOICE PDU.
	 */
    otx = ossItemIndexByName(world, ath, "value");
    otth = ossTypeHandleOfComponent(world, ath, otx);

    if (ret = ossEncode(world, PDU1_PDU, &pdu1, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));

	/*
	 * Create an OpenType component of the Attribute component of
	 * the CHOICE PDU.
	 */
    otcomp = NULL;
    err = ossPutEncodedValue(world, otth, encodedData, &otcomp);
    if (err) {
	ossterm(world);
	return err;
    }
    ossFreeBuf(world, encodedData.value);
	/*
	 * Add the OpenType component to the Attribute component of
	 * the CHOICE PDU.
	 */
    err = ossPutStructTypeValue(world, ath, otcomp, otx, &acomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a CHOICE PDU with the Attribute component just constructed.
	 */
    myPDU = NULL;
    err = ossPutStructTypeValue(world, pduth, acomp, ax, (void **)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Here we call ossPrintPDU() but it is also possible to call
	 * ossPrintDecodedValueOfPDU().
	 */
    ossPrint(world, "\n\tPDU constructed with IAAPI...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

    ossSetEncodingRules(world, OSS_BER);
    ossSetFlags(world, DEBUGPDU|AUTOMATIC_ENCDEC);
    encodedData.length = 0;
    encodedData.value = NULL;
    ossPrint(world, "\nEncoding the constructed PDU...\n\n");
    if (ret = ossEncode(world, PDU_PDU, myPDU, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));
    else {
        ossPrint(world, "\n\tEncoded constructed PDU...\n\n");
        ossPrintHex(world, encodedData.value, encodedData.length);
	decodedData = NULL;
	ossPrint(world, "\nDecoding the PDU...\n\n");
	if (ret = ossDecode(world, &pdu_num, &encodedData, (void **)&decodedData))
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decodedData);
	    ossFreePDU(world, pdu_num, decodedData);
	}
	ossFreeBuf(world, encodedData.value);
    }

    ossFreePDU(world, PDU_PDU, myPDU);

	/*
	 * Construct another PDU.  This time using ossPutOpenTypeDecodedValue().
	 */
    ossPrint(world, "\n\nConstructing another PDU with IAAPI...\n");
	/*
	 * Get the index and the type handle of the Attribute component of
	 * the CHOICE PDU.
	 */
    ax = ossItemIndexByName(world, pduth, "pdu3");
    ath = ossTypeHandleOfComponent(world, pduth, ax);

	/*
	 * Get the index and the type handle of the "id" component of
	 * the Attribute component of the CHOICE PDU.
	 */
    idx = ossItemIndexByName(world, ath, "id");
    idth = ossTypeHandleOfComponent(world, ath, idx);
	/*
	 * Create an "id" component of the Attribute component of
	 * the CHOICE PDU.
	 */
    idcomp = NULL;
    err = ossPutSimpleTypeValue(world, idth, "1", &idcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create an Attribute component of the CHOICE PDU with the
	 * "id" component.
	 */
    acomp = NULL;
    err = ossPutStructTypeValue(world, ath, idcomp, idx, &acomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Get the index and the type handle of the OpenType component of
	 * the Attribute component of the CHOICE PDU.
	 */
    otx = ossItemIndexByName(world, ath, "value");
    otth = ossTypeHandleOfComponent(world, ath, otx);

    err = buildUnencodedPDU(world, PDU1_PDU, (void **) &decodedData);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create an OpenType component of the Attribute component of
	 * the CHOICE PDU.
	 */
    otcomp = NULL;
    err = ossPutOpenTypeDecodedValue(world, otth, PDU1_PDU, decodedData, &otcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Add the OpenType component to the Attribute component of
	 * the CHOICE PDU.
	 */
    err = ossPutStructTypeValue(world, ath, otcomp, otx, &acomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a CHOICE PDU with the Attribute component just constructed.
	 */
    myPDU = NULL;
    err = ossPutStructTypeValue(world, pduth, acomp, ax, (void **) &myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

	/*
	 * Here we call ossPrintDecodedValueOfPDU() but it is also
	 * possible to call ossPrintPDU().
	 */
    ossPrint(world, "\n\tAnother PDU constructed with IAAPI...\n\n");
    ossPrintDecodedValueOfPDU(world, PDU_PDU, myPDU);

    ossPrint(world, "\nEncoding the constructed PDU...\n\n");
    encodedData.length = 0;
    encodedData.value = NULL;
    if (ret = ossEncode(world, PDU_PDU, myPDU, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));
    else {
        ossPrint(world, "\n\tEncoded constructed PDU...\n\n");
        ossPrintHex(world, encodedData.value, encodedData.length);
	decodedData = NULL;
	ossPrint(world, "\nDecoding the PDU...\n\n");
	if (ret = ossDecode(world, &pdu_num, &encodedData, (void **)&decodedData))
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	else {
	    void *value, *decoded;
	    OssBuf ossbuf = {0};
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decodedData);

	    err = ossGetDecodedValueOfComponent(world, pduth,
						decodedData, ax, &value);
	    err = ossGetDecodedValueOfComponent(world, ath,
						value, otx, &value);
	    err = ossGetOpenTypeValue(world,
				otth, value, &pdu_num, &decoded, &ossbuf);
	    ossPrint(world, "\n\tDecoded OpenType PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decoded);

	    ossFreePDU(world, pdu_num, decodedData);
	}
	ossFreeBuf(world, encodedData.value);
    }
	/*
	 * Here we call ossFreeDecodedValue() but it is also possible
	 * to call ossFreePDU().
	 */
    err = ossFreeDecodedValue(world, pduth, myPDU);

    ossterm(world);
    return ret;
}



static int buildUnencodedPDU(OssGlobal *world, int pdu_num, void **decodedData)
{

    TypeHndl pduth, oth, bth;
    unsigned int ox, bx;
    IAAPI_ERRTYPE err;
    void *pdu, *octetcomp, *bitcomp;


    pduth = ossTypeHandleOfPDU(world, pdu_num);

	/*
	 * Get the index and the type handle of the "data1" component of
	 * the PDU.
	 */
    ox = ossItemIndexByName(world, pduth, "data1");
    oth = ossTypeHandleOfComponent(world, pduth, ox);
	/*
	 * Create an OCTET STRING component of the PDU.
	 */
    octetcomp = NULL;
    err = ossPutSimpleTypeValue(world, oth, "'FFFFFF'H", &octetcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a PDU with the OCTET STRING component.
	 */
    pdu = NULL;
    err = ossPutStructTypeValue(world, pduth, octetcomp, ox, &pdu);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Get the index and the type handle of the "data2" component of
	 * the PDU.
	 */
    bx = ossItemIndexByName(world, pduth, "data2");
    bth = ossTypeHandleOfComponent(world, pduth, bx);
	/*
	 * Create a BIT STRING component of the PDU.
	 */
    err = ossPutSimpleTypeValue(world, bth, "'11111'B", &bitcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Add the BIT STRING component to the PDU.
	 */
    err = ossPutStructTypeValue(world, pduth, bitcomp, bx, &pdu);
    if (err) {
	ossterm(world);
	return err;
    }
    *decodedData = pdu;
    return 0;
}

